# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Mapping, cast
from typing_extensions import overload

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from ...._utils import extract_files, required_args, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.boxes import (
    f_info_params,
    f_list_params,
    f_read_params,
    f_write_params,
    f_exists_params,
    f_remove_params,
    f_rename_params,
)
from ....types.v1.boxes.file import File
from ....types.v1.boxes.f_info_response import FInfoResponse
from ....types.v1.boxes.f_list_response import FListResponse
from ....types.v1.boxes.f_read_response import FReadResponse
from ....types.v1.boxes.f_exists_response import FExistsResponse
from ....types.v1.boxes.f_remove_response import FRemoveResponse
from ....types.v1.boxes.f_rename_response import FRenameResponse

__all__ = ["FsResource", "AsyncFsResource"]


class FsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return FsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return FsResourceWithStreamingResponse(self)

    def list(
        self,
        box_id: str,
        *,
        path: str,
        depth: float | NotGiven = NOT_GIVEN,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FListResponse:
        """
        List box files

        Args:
          path: Target directory path in the box

          depth: Depth of the directory

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/fs/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "path": path,
                        "depth": depth,
                        "working_dir": working_dir,
                    },
                    f_list_params.FListParams,
                ),
            ),
            cast_to=FListResponse,
        )

    def exists(
        self,
        box_id: str,
        *,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FExistsResponse:
        """Check if file/dir exists

        Args:
          path: Target path in the box.

        If the path does not start with '/', the file/directory
              will be checked relative to the working directory

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            FExistsResponse,
            self._post(
                f"/boxes/{box_id}/fs/exists",
                body=maybe_transform(
                    {
                        "path": path,
                        "working_dir": working_dir,
                    },
                    f_exists_params.FExistsParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, FExistsResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def info(
        self,
        box_id: str,
        *,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FInfoResponse:
        """Get file/dir

        Args:
          path: Target path in the box.

        If the path does not start with '/', the file/directory
              will be checked relative to the working directory

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            FInfoResponse,
            self._get(
                f"/boxes/{box_id}/fs/info",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform(
                        {
                            "path": path,
                            "working_dir": working_dir,
                        },
                        f_info_params.FInfoParams,
                    ),
                ),
                cast_to=cast(Any, FInfoResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def read(
        self,
        box_id: str,
        *,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FReadResponse:
        """Read box file

        Args:
          path: Target path in the box.

        If the path does not start with '/', the file will be
              read from the working directory.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/fs/read",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "path": path,
                        "working_dir": working_dir,
                    },
                    f_read_params.FReadParams,
                ),
            ),
            cast_to=FReadResponse,
        )

    def remove(
        self,
        box_id: str,
        *,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FRemoveResponse:
        """Delete a file or dir.

        If target path is not exists, the delete will be failed.

        Args:
          path: Target path in the box. If the path does not start with '/', the file/directory
              will be deleted relative to the working directory. If the target path does not
              exist, the delete will fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._delete(
            f"/boxes/{box_id}/fs",
            body=maybe_transform(
                {
                    "path": path,
                    "working_dir": working_dir,
                },
                f_remove_params.FRemoveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FRemoveResponse,
        )

    def rename(
        self,
        box_id: str,
        *,
        new_path: str,
        old_path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FRenameResponse:
        """Rename a file or dir.

        If target newPath is already exists, the rename will be
        failed.

        Args:
          new_path: New path in the box. If the path does not start with '/', the file/directory
              will be renamed relative to the working directory. If the newPath already
              exists, the rename will fail.

          old_path: Old path in the box. If the path does not start with '/', the file/directory
              will be renamed relative to the working directory. If the oldPath does not
              exist, the rename will fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            FRenameResponse,
            self._post(
                f"/boxes/{box_id}/fs/rename",
                body=maybe_transform(
                    {
                        "new_path": new_path,
                        "old_path": old_path,
                        "working_dir": working_dir,
                    },
                    f_rename_params.FRenameParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, FRenameResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    def write(
        self,
        box_id: str,
        *,
        content: str,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> File:
        """Creates or overwrites a file.

        Creates necessary directories in the path if they
        don't exist. If target path is already exists, the write will be failed.

        Args:
          content: Content of the file (Max size: 512MB)

          path: Target path in the box. If the path does not start with '/', the file will be
              written relative to the working directory. Creates necessary directories in the
              path if they don't exist. If the target path already exists, the write will
              fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def write(
        self,
        box_id: str,
        *,
        content: FileTypes,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> File:
        """Creates or overwrites a file.

        Creates necessary directories in the path if they
        don't exist. If target path is already exists, the write will be failed.

        Args:
          content: Binary content of the file (Max file size: 512MB)

          path: Target path in the box. If the path does not start with '/', the file will be
              written relative to the working directory. Creates necessary directories in the
              path if they don't exist. If the target path already exists, the write will
              fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["content", "path"])
    def write(
        self,
        box_id: str,
        *,
        content: str | FileTypes,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> File:
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        body = deepcopy_minimal(
            {
                "content": content,
                "path": path,
                "working_dir": working_dir,
            }
        )
        if isinstance(content, str):
            return self._post(
                f"/boxes/{box_id}/fs/write",
                body=maybe_transform(body, f_write_params.FWriteParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=File,
            )
        else:
            files = extract_files(cast(Mapping[str, object], body), paths=[["content"]])
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
            return self._post(
                f"/boxes/{box_id}/fs/write",
                body=maybe_transform(body, f_write_params.FWriteParams),
                files=files,
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=File,
            )


class AsyncFsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AsyncFsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AsyncFsResourceWithStreamingResponse(self)

    async def list(
        self,
        box_id: str,
        *,
        path: str,
        depth: float | NotGiven = NOT_GIVEN,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FListResponse:
        """
        List box files

        Args:
          path: Target directory path in the box

          depth: Depth of the directory

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/fs/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "path": path,
                        "depth": depth,
                        "working_dir": working_dir,
                    },
                    f_list_params.FListParams,
                ),
            ),
            cast_to=FListResponse,
        )

    async def exists(
        self,
        box_id: str,
        *,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FExistsResponse:
        """Check if file/dir exists

        Args:
          path: Target path in the box.

        If the path does not start with '/', the file/directory
              will be checked relative to the working directory

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            FExistsResponse,
            await self._post(
                f"/boxes/{box_id}/fs/exists",
                body=await async_maybe_transform(
                    {
                        "path": path,
                        "working_dir": working_dir,
                    },
                    f_exists_params.FExistsParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, FExistsResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def info(
        self,
        box_id: str,
        *,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FInfoResponse:
        """Get file/dir

        Args:
          path: Target path in the box.

        If the path does not start with '/', the file/directory
              will be checked relative to the working directory

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            FInfoResponse,
            await self._get(
                f"/boxes/{box_id}/fs/info",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform(
                        {
                            "path": path,
                            "working_dir": working_dir,
                        },
                        f_info_params.FInfoParams,
                    ),
                ),
                cast_to=cast(Any, FInfoResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def read(
        self,
        box_id: str,
        *,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FReadResponse:
        """Read box file

        Args:
          path: Target path in the box.

        If the path does not start with '/', the file will be
              read from the working directory.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/fs/read",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "path": path,
                        "working_dir": working_dir,
                    },
                    f_read_params.FReadParams,
                ),
            ),
            cast_to=FReadResponse,
        )

    async def remove(
        self,
        box_id: str,
        *,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FRemoveResponse:
        """Delete a file or dir.

        If target path is not exists, the delete will be failed.

        Args:
          path: Target path in the box. If the path does not start with '/', the file/directory
              will be deleted relative to the working directory. If the target path does not
              exist, the delete will fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._delete(
            f"/boxes/{box_id}/fs",
            body=await async_maybe_transform(
                {
                    "path": path,
                    "working_dir": working_dir,
                },
                f_remove_params.FRemoveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FRemoveResponse,
        )

    async def rename(
        self,
        box_id: str,
        *,
        new_path: str,
        old_path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FRenameResponse:
        """Rename a file or dir.

        If target newPath is already exists, the rename will be
        failed.

        Args:
          new_path: New path in the box. If the path does not start with '/', the file/directory
              will be renamed relative to the working directory. If the newPath already
              exists, the rename will fail.

          old_path: Old path in the box. If the path does not start with '/', the file/directory
              will be renamed relative to the working directory. If the oldPath does not
              exist, the rename will fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return cast(
            FRenameResponse,
            await self._post(
                f"/boxes/{box_id}/fs/rename",
                body=await async_maybe_transform(
                    {
                        "new_path": new_path,
                        "old_path": old_path,
                        "working_dir": working_dir,
                    },
                    f_rename_params.FRenameParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, FRenameResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    async def write(
        self,
        box_id: str,
        *,
        content: str,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> File:
        """Creates or overwrites a file.

        Creates necessary directories in the path if they
        don't exist. If target path is already exists, the write will be failed.

        Args:
          content: Content of the file (Max size: 512MB)

          path: Target path in the box. If the path does not start with '/', the file will be
              written relative to the working directory. Creates necessary directories in the
              path if they don't exist. If the target path already exists, the write will
              fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def write(
        self,
        box_id: str,
        *,
        content: FileTypes,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> File:
        """Creates or overwrites a file.

        Creates necessary directories in the path if they
        don't exist. If target path is already exists, the write will be failed.

        Args:
          content: Binary content of the file (Max file size: 512MB)

          path: Target path in the box. If the path does not start with '/', the file will be
              written relative to the working directory. Creates necessary directories in the
              path if they don't exist. If the target path already exists, the write will
              fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["content", "path"])
    async def write(
        self,
        box_id: str,
        *,
        content: str | FileTypes,
        path: str,
        working_dir: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> File:
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        body = deepcopy_minimal(
            {
                "content": content,
                "path": path,
                "working_dir": working_dir,
            }
        )
        if isinstance(content, str):
            return await self._post(
                f"/boxes/{box_id}/fs/write",
                body=await async_maybe_transform(body, f_write_params.FWriteParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=File,
            )
        else:
            files = extract_files(cast(Mapping[str, object], body), paths=[["content"]])
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
            return await self._post(
                f"/boxes/{box_id}/fs/write",
                body=await async_maybe_transform(body, f_write_params.FWriteParams),
                files=files,
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=File,
            )


class FsResourceWithRawResponse:
    def __init__(self, fs: FsResource) -> None:
        self._fs = fs

        self.list = to_raw_response_wrapper(
            fs.list,
        )
        self.exists = to_raw_response_wrapper(
            fs.exists,
        )
        self.info = to_raw_response_wrapper(
            fs.info,
        )
        self.read = to_raw_response_wrapper(
            fs.read,
        )
        self.remove = to_raw_response_wrapper(
            fs.remove,
        )
        self.rename = to_raw_response_wrapper(
            fs.rename,
        )
        self.write = to_raw_response_wrapper(
            fs.write,
        )


class AsyncFsResourceWithRawResponse:
    def __init__(self, fs: AsyncFsResource) -> None:
        self._fs = fs

        self.list = async_to_raw_response_wrapper(
            fs.list,
        )
        self.exists = async_to_raw_response_wrapper(
            fs.exists,
        )
        self.info = async_to_raw_response_wrapper(
            fs.info,
        )
        self.read = async_to_raw_response_wrapper(
            fs.read,
        )
        self.remove = async_to_raw_response_wrapper(
            fs.remove,
        )
        self.rename = async_to_raw_response_wrapper(
            fs.rename,
        )
        self.write = async_to_raw_response_wrapper(
            fs.write,
        )


class FsResourceWithStreamingResponse:
    def __init__(self, fs: FsResource) -> None:
        self._fs = fs

        self.list = to_streamed_response_wrapper(
            fs.list,
        )
        self.exists = to_streamed_response_wrapper(
            fs.exists,
        )
        self.info = to_streamed_response_wrapper(
            fs.info,
        )
        self.read = to_streamed_response_wrapper(
            fs.read,
        )
        self.remove = to_streamed_response_wrapper(
            fs.remove,
        )
        self.rename = to_streamed_response_wrapper(
            fs.rename,
        )
        self.write = to_streamed_response_wrapper(
            fs.write,
        )


class AsyncFsResourceWithStreamingResponse:
    def __init__(self, fs: AsyncFsResource) -> None:
        self._fs = fs

        self.list = async_to_streamed_response_wrapper(
            fs.list,
        )
        self.exists = async_to_streamed_response_wrapper(
            fs.exists,
        )
        self.info = async_to_streamed_response_wrapper(
            fs.info,
        )
        self.read = async_to_streamed_response_wrapper(
            fs.read,
        )
        self.remove = async_to_streamed_response_wrapper(
            fs.remove,
        )
        self.rename = async_to_streamed_response_wrapper(
            fs.rename,
        )
        self.write = async_to_streamed_response_wrapper(
            fs.write,
        )



import json
import asyncio
import threading
from typing import Any, Dict, List, Union, Callable, Optional
from dataclasses import dataclass
from typing_extensions import Literal

import websocket
from websocket import WebSocket

from gbox_sdk._types import NOT_GIVEN, NotGiven


@dataclass
class WebSocketResult:
    """Result from WebSocket command execution"""

    exit_code: int
    stdout: str
    stderr: str


class WebSocketClient:
    """WebSocket client for streaming command execution"""

    def __init__(self, websocket_url: str, api_key: str):
        self.websocket_url = websocket_url
        self.api_key = api_key
        self.headers = {
            "Authorization": f"Bearer {self.api_key}",
        }

    async def execute_command(
        self,
        commands: Union[List[str], str],
        on_stdout: Optional[Callable[[str], None]] = None,
        on_stderr: Optional[Callable[[str], None]] = None,
        envs: Union[Dict[str, str], NotGiven] = NOT_GIVEN,
        api_timeout: Union[str, NotGiven] = NOT_GIVEN,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
    ) -> WebSocketResult:
        """
        Execute commands via WebSocket with streaming output

        Args:
            commands: Commands to execute
            on_stdout: Callback for stdout data
            on_stderr: Callback for stderr data
            envs: Environment variables
            api_timeout: API timeout
            working_dir: Working directory

        Returns:
            WebSocketResult with execution results
        """
        command_obj: Dict[str, Any] = {
            "commands": commands if isinstance(commands, list) else [commands],
        }

        payload: Dict[str, Any] = {
            "command": command_obj,
        }

        if envs is not NOT_GIVEN:
            payload["envs"] = envs
        if api_timeout is not NOT_GIVEN:
            payload["timeout"] = api_timeout
        if working_dir is not NOT_GIVEN:
            payload["workingDir"] = working_dir

        return await self._execute_via_websocket(payload, on_stdout, on_stderr)

    async def run_code(
        self,
        code: str,
        on_stdout: Optional[Callable[[str], None]] = None,
        on_stderr: Optional[Callable[[str], None]] = None,
        argv: Union[List[str], NotGiven] = NOT_GIVEN,
        envs: Union[Dict[str, str], NotGiven] = NOT_GIVEN,
        language: Union[Literal["bash", "python", "typescript"], NotGiven] = NOT_GIVEN,
        api_timeout: Union[str, NotGiven] = NOT_GIVEN,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
    ) -> WebSocketResult:
        """
        Run code via WebSocket with streaming output

        Args:
            code: Code to execute
            on_stdout: Callback for stdout data
            on_stderr: Callback for stderr data
            argv: Arguments to pass to the code
            envs: Environment variables
            language: Programming language (bash, python, typescript)
            api_timeout: API timeout
            working_dir: Working directory

        Returns:
            WebSocketResult with execution results
        """
        run_code_obj: Dict[str, Any] = {
            "code": code,
        }

        if language is not NOT_GIVEN:
            run_code_obj["language"] = language

        payload: Dict[str, Any] = {
            "runCode": run_code_obj,
        }

        if argv is not NOT_GIVEN:
            payload["argv"] = argv
        if envs is not NOT_GIVEN:
            payload["envs"] = envs
        if api_timeout is not NOT_GIVEN:
            payload["timeout"] = api_timeout
        if working_dir is not NOT_GIVEN:
            payload["workingDir"] = working_dir

        return await self._execute_via_websocket(payload, on_stdout, on_stderr)

    async def _execute_via_websocket(
        self,
        payload: Dict[str, Any],
        on_stdout: Optional[Callable[[str], None]] = None,
        on_stderr: Optional[Callable[[str], None]] = None,
    ) -> WebSocketResult:
        """Execute command via WebSocket with streaming output"""

        def run_websocket() -> WebSocketResult:
            stdout = ""
            stderr = ""
            exit_code = 0
            result_event = threading.Event()
            result: Optional[WebSocketResult] = None

            def on_message(ws: WebSocket, message: str) -> None:
                nonlocal stdout, stderr, exit_code, result
                try:
                    data = json.loads(message)
                    if data.get("event") == "stdout":
                        output = data.get("data", "")
                        stdout += output
                        if on_stdout:
                            on_stdout(output)
                    elif data.get("event") == "stderr":
                        output = data.get("data", "")
                        stderr += output
                        if on_stderr:
                            on_stderr(output)
                    elif data.get("event") == "end":
                        exit_code = data.get("exitCode", 0)
                        ws.close()
                except Exception as e:
                    result = WebSocketResult(exit_code=1, stdout="", stderr=f"Failed to parse WebSocket message: {e}")
                    result_event.set()
                    ws.close()

            def on_error(_ws: WebSocket, error: str) -> None:
                nonlocal result
                result = WebSocketResult(exit_code=1, stdout="", stderr=f"WebSocket error: {error}")
                result_event.set()

            def on_close(_ws: WebSocket, _close_status_code: int, _close_msg: str) -> None:
                nonlocal result
                if result is None:
                    result = WebSocketResult(exit_code=exit_code, stdout=stdout, stderr=stderr)
                result_event.set()

            def on_open(ws: WebSocket) -> None:
                try:
                    ws.send(json.dumps(payload))
                except Exception as e:
                    nonlocal result
                    result = WebSocketResult(exit_code=1, stdout="", stderr=f"Failed to send payload: {e}")
                    result_event.set()
                    ws.close()

            ws = websocket.WebSocketApp(
                self.websocket_url,
                header=self.headers,
                on_open=on_open,
                on_message=on_message,
                on_error=on_error,
                on_close=on_close,
            )

            ws.run_forever()  # type: ignore[attr-defined]
            result_event.wait()
            if result is None:
                result = WebSocketResult(exit_code=1, stdout="", stderr="No result received")
            return result

        loop = asyncio.get_running_loop()
        result = await loop.run_in_executor(None, run_websocket)
        return result



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Mapping, cast
from typing_extensions import Literal, overload

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven, FileTypes
from ...._utils import extract_files, required_args, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.boxes import (
    android_open_params,
    android_install_params,
    android_restart_params,
    android_restore_params,
    android_list_pkg_params,
    android_uninstall_params,
    android_list_pkg_simple_params,
)
from ....types.v1.boxes.android_app import AndroidApp
from ....types.v1.boxes.android_pkg import AndroidPkg
from ....types.v1.boxes.android_install_response import AndroidInstallResponse
from ....types.v1.boxes.android_list_app_response import AndroidListAppResponse
from ....types.v1.boxes.android_list_pkg_response import AndroidListPkgResponse
from ....types.v1.boxes.android_list_activities_response import AndroidListActivitiesResponse
from ....types.v1.boxes.android_list_pkg_simple_response import AndroidListPkgSimpleResponse
from ....types.v1.boxes.android_get_connect_address_response import AndroidGetConnectAddressResponse

__all__ = ["AndroidResource", "AsyncAndroidResource"]


class AndroidResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AndroidResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AndroidResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AndroidResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AndroidResourceWithStreamingResponse(self)

    def backup(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BinaryAPIResponse:
        """
        Backup

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/backup",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def backup_all(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BinaryAPIResponse:
        """
        Backup all

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/backup-all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def close(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Close app

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/close",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def close_all(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Close all apps

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/close-all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidPkg:
        """
        Get pkg

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return self._get(
            f"/boxes/{box_id}/android/packages/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidPkg,
        )

    def get_app(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidApp:
        """
        Get installed app info by package name

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return self._get(
            f"/boxes/{box_id}/android/apps/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidApp,
        )

    def get_connect_address(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidGetConnectAddressResponse:
        """
        Get connect address

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/android/connect-address",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidGetConnectAddressResponse,
        )

    @overload
    def install(
        self,
        box_id: str,
        *,
        apk: FileTypes,
        open: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        """
        Install app

        Args:
          apk: APK file or ZIP archive to install (max file size: 512MB).

              **Single APK mode:**

              - Upload a single APK file (e.g., app.apk)
              - System will automatically detect and install as single APK

              **Multi-APK mode (automatically detected):**

              - Upload a ZIP archive containing multiple APK files
              - System will automatically detect ZIP format and install all APKs inside
              - ZIP filename example: com.reddit.frontpage-gplay.zip
              - ZIP contents example:

              com.reddit.frontpage-gplay.zip └── com.reddit.frontpage-gplay/ (folder) ├──
              reddit-base.apk (base APK) ├── reddit-arm64.apk (architecture-specific) ├──
              reddit-en.apk (language pack) └── reddit-mdpi.apk (density-specific resources)

              This is commonly used for split APKs where different components are separated by
              architecture, language, or screen density.

          open: Whether to open the app after installation. Will find and launch the launcher
              activity of the installed app. If there are multiple launcher activities, only
              one will be opened. If the installed APK has no launcher activity, this
              parameter will have no effect.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def install(
        self,
        box_id: str,
        *,
        apk: str,
        open: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        """
        Install app

        Args:
          apk: HTTP URL to download APK file or ZIP archive (max file size: 512MB).

              **Single APK mode (automatically detected):**

              - Provide URL to a single APK file
              - System will automatically detect .apk extension and install as single APK
              - Example: https://example.com/app.apk

              **Multi-APK mode (automatically detected):**

              - Provide URL to a ZIP archive containing multiple APK files
              - System will automatically detect .zip extension and install all APKs inside
              - ZIP filename example: com.reddit.frontpage-gplay.zip
              - ZIP contents example:

              com.reddit.frontpage-gplay.zip └── com.reddit.frontpage-gplay/ (folder) ├──
              reddit-base.apk (base APK) ├── reddit-arm64.apk (architecture-specific) ├──
              reddit-en.apk (language pack) └── reddit-mdpi.apk (density-specific resources)

              - Example URL: https://example.com/com.reddit.frontpage-gplay.zip

              This is commonly used for split APKs where different components are separated by
              architecture, language, or screen density.

          open: Whether to open the app after installation. Will find and launch the launcher
              activity of the installed app. If there are multiple launcher activities, only
              one will be opened. If the installed APK has no launcher activity, this
              parameter will have no effect.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["apk"])
    def install(
        self,
        box_id: str,
        *,
        apk: FileTypes | str,
        open: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        body = deepcopy_minimal(
            {
                "apk": apk,
                "open": open,
            }
        )
        if isinstance(apk, str):
            return self._post(
                f"/boxes/{box_id}/android/packages",
                body=maybe_transform(body, android_install_params.AndroidInstallParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=AndroidInstallResponse,
            )
        else:
            files = extract_files(cast(Mapping[str, object], body), paths=[["apk"]])
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
            return self._post(
                f"/boxes/{box_id}/android/packages",
                body=maybe_transform(body, android_install_params.AndroidInstallParams),
                files=files,
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=AndroidInstallResponse,
            )

    def list_activities(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListActivitiesResponse:
        """
        Get pkg activities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return self._get(
            f"/boxes/{box_id}/android/packages/{package_name}/activities",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidListActivitiesResponse,
        )

    def list_app(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListAppResponse:
        """
        List all installed apps on the launcher

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/android/apps",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidListAppResponse,
        )

    def list_pkg(
        self,
        box_id: str,
        *,
        pkg_type: List[Literal["system", "thirdParty"]] | NotGiven = NOT_GIVEN,
        running_filter: List[Literal["running", "notRunning"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListPkgResponse:
        """Retrieve detailed information for all installed pkg.

        This endpoint provides
        comprehensive pkg details

        Args:
          pkg_type: system or thirdParty, default is thirdParty

          running_filter: Filter pkgs by running status: running (show only running pkgs), notRunning
              (show only non-running pkgs). Default is all

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/android/packages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "pkg_type": pkg_type,
                        "running_filter": running_filter,
                    },
                    android_list_pkg_params.AndroidListPkgParams,
                ),
            ),
            cast_to=AndroidListPkgResponse,
        )

    def list_pkg_simple(
        self,
        box_id: str,
        *,
        pkg_type: List[Literal["system", "thirdParty"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListPkgSimpleResponse:
        """A faster endpoint to quickly retrieve basic pkg information.

        This API provides
        better performance for scenarios where you need to get essential pkg details
        quickly

        Args:
          pkg_type: system or thirdParty, default is thirdParty

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/android/packages/simple",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"pkg_type": pkg_type}, android_list_pkg_simple_params.AndroidListPkgSimpleParams
                ),
            ),
            cast_to=AndroidListPkgSimpleResponse,
        )

    def open(
        self,
        package_name: str,
        *,
        box_id: str,
        activity_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Open app

        Args:
          activity_name: Activity name, default is the main activity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/open",
            body=maybe_transform({"activity_name": activity_name}, android_open_params.AndroidOpenParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def restart(
        self,
        package_name: str,
        *,
        box_id: str,
        activity_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Restart app

        Args:
          activity_name: Activity name, default is the main activity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/restart",
            body=maybe_transform({"activity_name": activity_name}, android_restart_params.AndroidRestartParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def restore(
        self,
        box_id: str,
        *,
        backup: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Restore

        Args:
          backup: Backup file to restore (max file size: 100MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/android/packages/restore",
            body=maybe_transform({"backup": backup}, android_restore_params.AndroidRestoreParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def uninstall(
        self,
        package_name: str,
        *,
        box_id: str,
        keep_data: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Uninstall app

        Args:
          keep_data: uninstalls the pkg while retaining the data/cache

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/boxes/{box_id}/android/packages/{package_name}",
            body=maybe_transform({"keep_data": keep_data}, android_uninstall_params.AndroidUninstallParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncAndroidResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAndroidResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAndroidResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAndroidResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AsyncAndroidResourceWithStreamingResponse(self)

    async def backup(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncBinaryAPIResponse:
        """
        Backup

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/backup",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def backup_all(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncBinaryAPIResponse:
        """
        Backup all

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/backup-all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def close(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Close app

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/close",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def close_all(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Close all apps

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/close-all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidPkg:
        """
        Get pkg

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return await self._get(
            f"/boxes/{box_id}/android/packages/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidPkg,
        )

    async def get_app(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidApp:
        """
        Get installed app info by package name

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return await self._get(
            f"/boxes/{box_id}/android/apps/{package_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidApp,
        )

    async def get_connect_address(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidGetConnectAddressResponse:
        """
        Get connect address

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/android/connect-address",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidGetConnectAddressResponse,
        )

    @overload
    async def install(
        self,
        box_id: str,
        *,
        apk: FileTypes,
        open: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        """
        Install app

        Args:
          apk: APK file or ZIP archive to install (max file size: 512MB).

              **Single APK mode:**

              - Upload a single APK file (e.g., app.apk)
              - System will automatically detect and install as single APK

              **Multi-APK mode (automatically detected):**

              - Upload a ZIP archive containing multiple APK files
              - System will automatically detect ZIP format and install all APKs inside
              - ZIP filename example: com.reddit.frontpage-gplay.zip
              - ZIP contents example:

              com.reddit.frontpage-gplay.zip └── com.reddit.frontpage-gplay/ (folder) ├──
              reddit-base.apk (base APK) ├── reddit-arm64.apk (architecture-specific) ├──
              reddit-en.apk (language pack) └── reddit-mdpi.apk (density-specific resources)

              This is commonly used for split APKs where different components are separated by
              architecture, language, or screen density.

          open: Whether to open the app after installation. Will find and launch the launcher
              activity of the installed app. If there are multiple launcher activities, only
              one will be opened. If the installed APK has no launcher activity, this
              parameter will have no effect.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def install(
        self,
        box_id: str,
        *,
        apk: str,
        open: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        """
        Install app

        Args:
          apk: HTTP URL to download APK file or ZIP archive (max file size: 512MB).

              **Single APK mode (automatically detected):**

              - Provide URL to a single APK file
              - System will automatically detect .apk extension and install as single APK
              - Example: https://example.com/app.apk

              **Multi-APK mode (automatically detected):**

              - Provide URL to a ZIP archive containing multiple APK files
              - System will automatically detect .zip extension and install all APKs inside
              - ZIP filename example: com.reddit.frontpage-gplay.zip
              - ZIP contents example:

              com.reddit.frontpage-gplay.zip └── com.reddit.frontpage-gplay/ (folder) ├──
              reddit-base.apk (base APK) ├── reddit-arm64.apk (architecture-specific) ├──
              reddit-en.apk (language pack) └── reddit-mdpi.apk (density-specific resources)

              - Example URL: https://example.com/com.reddit.frontpage-gplay.zip

              This is commonly used for split APKs where different components are separated by
              architecture, language, or screen density.

          open: Whether to open the app after installation. Will find and launch the launcher
              activity of the installed app. If there are multiple launcher activities, only
              one will be opened. If the installed APK has no launcher activity, this
              parameter will have no effect.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["apk"])
    async def install(
        self,
        box_id: str,
        *,
        apk: FileTypes | str,
        open: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidInstallResponse:
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        body = deepcopy_minimal(
            {
                "apk": apk,
                "open": open,
            }
        )
        if isinstance(apk, str):
            return await self._post(
                f"/boxes/{box_id}/android/packages",
                body=await async_maybe_transform(body, android_install_params.AndroidInstallParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=AndroidInstallResponse,
            )
        else:
            files = extract_files(cast(Mapping[str, object], body), paths=[["apk"]])
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
            return await self._post(
                f"/boxes/{box_id}/android/packages",
                body=await async_maybe_transform(body, android_install_params.AndroidInstallParams),
                files=files,
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=AndroidInstallResponse,
            )

    async def list_activities(
        self,
        package_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListActivitiesResponse:
        """
        Get pkg activities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        return await self._get(
            f"/boxes/{box_id}/android/packages/{package_name}/activities",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidListActivitiesResponse,
        )

    async def list_app(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListAppResponse:
        """
        List all installed apps on the launcher

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/android/apps",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AndroidListAppResponse,
        )

    async def list_pkg(
        self,
        box_id: str,
        *,
        pkg_type: List[Literal["system", "thirdParty"]] | NotGiven = NOT_GIVEN,
        running_filter: List[Literal["running", "notRunning"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListPkgResponse:
        """Retrieve detailed information for all installed pkg.

        This endpoint provides
        comprehensive pkg details

        Args:
          pkg_type: system or thirdParty, default is thirdParty

          running_filter: Filter pkgs by running status: running (show only running pkgs), notRunning
              (show only non-running pkgs). Default is all

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/android/packages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "pkg_type": pkg_type,
                        "running_filter": running_filter,
                    },
                    android_list_pkg_params.AndroidListPkgParams,
                ),
            ),
            cast_to=AndroidListPkgResponse,
        )

    async def list_pkg_simple(
        self,
        box_id: str,
        *,
        pkg_type: List[Literal["system", "thirdParty"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AndroidListPkgSimpleResponse:
        """A faster endpoint to quickly retrieve basic pkg information.

        This API provides
        better performance for scenarios where you need to get essential pkg details
        quickly

        Args:
          pkg_type: system or thirdParty, default is thirdParty

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/android/packages/simple",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"pkg_type": pkg_type}, android_list_pkg_simple_params.AndroidListPkgSimpleParams
                ),
            ),
            cast_to=AndroidListPkgSimpleResponse,
        )

    async def open(
        self,
        package_name: str,
        *,
        box_id: str,
        activity_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Open app

        Args:
          activity_name: Activity name, default is the main activity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/open",
            body=await async_maybe_transform({"activity_name": activity_name}, android_open_params.AndroidOpenParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def restart(
        self,
        package_name: str,
        *,
        box_id: str,
        activity_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Restart app

        Args:
          activity_name: Activity name, default is the main activity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/{package_name}/restart",
            body=await async_maybe_transform(
                {"activity_name": activity_name}, android_restart_params.AndroidRestartParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def restore(
        self,
        box_id: str,
        *,
        backup: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Restore

        Args:
          backup: Backup file to restore (max file size: 100MB)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/android/packages/restore",
            body=await async_maybe_transform({"backup": backup}, android_restore_params.AndroidRestoreParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def uninstall(
        self,
        package_name: str,
        *,
        box_id: str,
        keep_data: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Uninstall app

        Args:
          keep_data: uninstalls the pkg while retaining the data/cache

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not package_name:
            raise ValueError(f"Expected a non-empty value for `package_name` but received {package_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/boxes/{box_id}/android/packages/{package_name}",
            body=await async_maybe_transform({"keep_data": keep_data}, android_uninstall_params.AndroidUninstallParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AndroidResourceWithRawResponse:
    def __init__(self, android: AndroidResource) -> None:
        self._android = android

        self.backup = to_custom_raw_response_wrapper(
            android.backup,
            BinaryAPIResponse,
        )
        self.backup_all = to_custom_raw_response_wrapper(
            android.backup_all,
            BinaryAPIResponse,
        )
        self.close = to_raw_response_wrapper(
            android.close,
        )
        self.close_all = to_raw_response_wrapper(
            android.close_all,
        )
        self.get = to_raw_response_wrapper(
            android.get,
        )
        self.get_app = to_raw_response_wrapper(
            android.get_app,
        )
        self.get_connect_address = to_raw_response_wrapper(
            android.get_connect_address,
        )
        self.install = to_raw_response_wrapper(
            android.install,
        )
        self.list_activities = to_raw_response_wrapper(
            android.list_activities,
        )
        self.list_app = to_raw_response_wrapper(
            android.list_app,
        )
        self.list_pkg = to_raw_response_wrapper(
            android.list_pkg,
        )
        self.list_pkg_simple = to_raw_response_wrapper(
            android.list_pkg_simple,
        )
        self.open = to_raw_response_wrapper(
            android.open,
        )
        self.restart = to_raw_response_wrapper(
            android.restart,
        )
        self.restore = to_raw_response_wrapper(
            android.restore,
        )
        self.uninstall = to_raw_response_wrapper(
            android.uninstall,
        )


class AsyncAndroidResourceWithRawResponse:
    def __init__(self, android: AsyncAndroidResource) -> None:
        self._android = android

        self.backup = async_to_custom_raw_response_wrapper(
            android.backup,
            AsyncBinaryAPIResponse,
        )
        self.backup_all = async_to_custom_raw_response_wrapper(
            android.backup_all,
            AsyncBinaryAPIResponse,
        )
        self.close = async_to_raw_response_wrapper(
            android.close,
        )
        self.close_all = async_to_raw_response_wrapper(
            android.close_all,
        )
        self.get = async_to_raw_response_wrapper(
            android.get,
        )
        self.get_app = async_to_raw_response_wrapper(
            android.get_app,
        )
        self.get_connect_address = async_to_raw_response_wrapper(
            android.get_connect_address,
        )
        self.install = async_to_raw_response_wrapper(
            android.install,
        )
        self.list_activities = async_to_raw_response_wrapper(
            android.list_activities,
        )
        self.list_app = async_to_raw_response_wrapper(
            android.list_app,
        )
        self.list_pkg = async_to_raw_response_wrapper(
            android.list_pkg,
        )
        self.list_pkg_simple = async_to_raw_response_wrapper(
            android.list_pkg_simple,
        )
        self.open = async_to_raw_response_wrapper(
            android.open,
        )
        self.restart = async_to_raw_response_wrapper(
            android.restart,
        )
        self.restore = async_to_raw_response_wrapper(
            android.restore,
        )
        self.uninstall = async_to_raw_response_wrapper(
            android.uninstall,
        )


class AndroidResourceWithStreamingResponse:
    def __init__(self, android: AndroidResource) -> None:
        self._android = android

        self.backup = to_custom_streamed_response_wrapper(
            android.backup,
            StreamedBinaryAPIResponse,
        )
        self.backup_all = to_custom_streamed_response_wrapper(
            android.backup_all,
            StreamedBinaryAPIResponse,
        )
        self.close = to_streamed_response_wrapper(
            android.close,
        )
        self.close_all = to_streamed_response_wrapper(
            android.close_all,
        )
        self.get = to_streamed_response_wrapper(
            android.get,
        )
        self.get_app = to_streamed_response_wrapper(
            android.get_app,
        )
        self.get_connect_address = to_streamed_response_wrapper(
            android.get_connect_address,
        )
        self.install = to_streamed_response_wrapper(
            android.install,
        )
        self.list_activities = to_streamed_response_wrapper(
            android.list_activities,
        )
        self.list_app = to_streamed_response_wrapper(
            android.list_app,
        )
        self.list_pkg = to_streamed_response_wrapper(
            android.list_pkg,
        )
        self.list_pkg_simple = to_streamed_response_wrapper(
            android.list_pkg_simple,
        )
        self.open = to_streamed_response_wrapper(
            android.open,
        )
        self.restart = to_streamed_response_wrapper(
            android.restart,
        )
        self.restore = to_streamed_response_wrapper(
            android.restore,
        )
        self.uninstall = to_streamed_response_wrapper(
            android.uninstall,
        )


class AsyncAndroidResourceWithStreamingResponse:
    def __init__(self, android: AsyncAndroidResource) -> None:
        self._android = android

        self.backup = async_to_custom_streamed_response_wrapper(
            android.backup,
            AsyncStreamedBinaryAPIResponse,
        )
        self.backup_all = async_to_custom_streamed_response_wrapper(
            android.backup_all,
            AsyncStreamedBinaryAPIResponse,
        )
        self.close = async_to_streamed_response_wrapper(
            android.close,
        )
        self.close_all = async_to_streamed_response_wrapper(
            android.close_all,
        )
        self.get = async_to_streamed_response_wrapper(
            android.get,
        )
        self.get_app = async_to_streamed_response_wrapper(
            android.get_app,
        )
        self.get_connect_address = async_to_streamed_response_wrapper(
            android.get_connect_address,
        )
        self.install = async_to_streamed_response_wrapper(
            android.install,
        )
        self.list_activities = async_to_streamed_response_wrapper(
            android.list_activities,
        )
        self.list_app = async_to_streamed_response_wrapper(
            android.list_app,
        )
        self.list_pkg = async_to_streamed_response_wrapper(
            android.list_pkg,
        )
        self.list_pkg_simple = async_to_streamed_response_wrapper(
            android.list_pkg_simple,
        )
        self.open = async_to_streamed_response_wrapper(
            android.open,
        )
        self.restart = async_to_streamed_response_wrapper(
            android.restart,
        )
        self.restore = async_to_streamed_response_wrapper(
            android.restore,
        )
        self.uninstall = async_to_streamed_response_wrapper(
            android.uninstall,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .linux_box import LinuxBox
from .android_box import AndroidBox

__all__ = ["BoxListResponse", "Data"]

Data: TypeAlias = Union[LinuxBox, AndroidBox]


class BoxListResponse(BaseModel):
    data: List[Data]
    """A box instance that can be either Linux or Android type"""

    page: int
    """Page number"""

    page_size: int = FieldInfo(alias="pageSize")
    """Page size"""

    total: int
    """Total number of items"""



from typing import List, Union, Optional

from gbox_sdk._types import NOT_GIVEN, NotGiven
from gbox_sdk._client import GboxClient
from gbox_sdk.types.v1.boxes.dir import Dir
from gbox_sdk.types.v1.boxes.file import File
from gbox_sdk.types.v1.boxes.f_write_params import FileTypes
from gbox_sdk.types.v1.boxes.f_list_response import Data, FListResponse
from gbox_sdk.types.v1.boxes.f_read_response import FReadResponse
from gbox_sdk.types.v1.boxes.f_exists_response import FExistsResponse
from gbox_sdk.types.v1.boxes.f_remove_response import FRemoveResponse
from gbox_sdk.types.v1.boxes.f_rename_response import FRenameResponse


class FileSystemOperator:
    """
    Operator for file system operations within a box.

    Provides methods to list, read, write, remove, check existence, rename, and retrieve file or directory information in a box.

    Args:
        client (GboxClient): The Gbox client instance.
        box_id (str): The ID of the box to operate on.
    """

    def __init__(self, client: GboxClient, box_id: str):
        self.client = client
        self.box_id = box_id

    def list_info(
        self,
        path: str,
        *,
        depth: Union[float, NotGiven] = NOT_GIVEN,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
    ) -> FListResponse:
        """
        Get detailed information about files and directories at a given path or with given parameters.

        Args:
            path: Target directory path in the box

            depth: Depth of the directory

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            FListResponse: The response containing file/directory information.

        Example:
            >>> box.file_system.list_info(path="/path/to/directory", depth=1, working_dir="/path/to/working_dir")
            >>> box.file_system.list_info("/path/to/directory")
        """
        return self.client.v1.boxes.fs.list(box_id=self.box_id, path=path, depth=depth, working_dir=working_dir)

    def list(
        self,
        path: str,
        *,
        depth: Union[float, NotGiven] = NOT_GIVEN,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
    ) -> List[Union["FileOperator", "DirectoryOperator"]]:
        """
        List files and directories at a given path or with given parameters, returning operator objects.

        Args:
            path: Target directory path in the box

            depth: Depth of the directory

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            List[Union[FileOperator, DirectoryOperator]]: List of file or directory operator objects.

        Example:
            >>> box.file_system.list(path="/path/to/directory", depth=1, working_dir="/path/to/working_dir")
            >>> box.file_system.list("/path/to/directory")
        """
        res = self.list_info(path, depth=depth, working_dir=working_dir)
        return [self._data_to_operator(r) for r in res.data]

    def read(
        self,
        path: str,
        *,
        working_dir: Union[str, NotGiven] = NOT_GIVEN,
    ) -> FReadResponse:
        """
        Read the content of a file.

        Args:
            path: Target path in the box. If the path does not start with '/', the file will be
                read from the working directory.

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            FReadResponse: The response containing file content.

        Example:
            >>> box.file_system.read(path="/path/to/file", working_dir="/path/to/working_dir")
            >>> box.file_system.read("/path/to/file")
        """
        return self.client.v1.boxes.fs.read(box_id=self.box_id, path=path, working_dir=working_dir)

    def write(
        self, *, content: Union[str, FileTypes], path: str, working_dir: Union[str, NotGiven] = NOT_GIVEN
    ) -> "FileOperator":
        """
        Write content to a file (text or binary).

        Args:
            content: Content of the file (Max size: 512MB)

            path: Target path in the box. If the path does not start with '/', the file will be
                written relative to the working directory. Creates necessary directories in the
                path if they don't exist. If the target path already exists, the write will
                fail.

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            FileOperator: The file operator for the written file.

        Example:
            >>> box.file_system.write(content="Hello, World!", path="/path/to/file", working_dir="/path/to/working_dir")
            >>> box.file_system.write(content="Hello, World!", path="/path/to/file")
        """
        res = self.client.v1.boxes.fs.write(
            box_id=self.box_id, content=content, path=path, working_dir=working_dir if working_dir else NOT_GIVEN
        )

        # Convert FWriteResponse to DataFile format for FileOperator
        data_file = File(
            path=res.path, type="file", mode=res.mode, name=res.name, size=res.size, lastModified=res.last_modified
        )

        return FileOperator(self.client, self.box_id, data_file)

    def remove(self, path: str, *, working_dir: Union[str, NotGiven] = NOT_GIVEN) -> FRemoveResponse:
        """
        Remove a file or directory.

        Args:
            path: Target path in the box. If the path does not start with '/', the file/directory
                will be checked relative to the working directory

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            FRemoveResponse: The response after removing the file or directory.

        Example:
            >>> box.file_system.remove(path="/path/to/file", working_dir="/path/to/working_dir")
            >>> box.file_system.remove("/path/to/file")
        """
        return self.client.v1.boxes.fs.remove(box_id=self.box_id, path=path, working_dir=working_dir)

    def exists(self, path: str, *, working_dir: Union[str, NotGiven] = NOT_GIVEN) -> FExistsResponse:
        """
        Check if a file or directory exists.

        Args:
            path: Target path in the box. If the path does not start with '/', the file/directory
                will be checked relative to the working directory

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            FExistsResponse: The response indicating existence.

        Example:
            >>> box.file_system.exists(path="/path/to/file", working_dir="/path/to/working_dir")
            >>> box.file_system.exists("/path/to/file")
        """
        return self.client.v1.boxes.fs.exists(box_id=self.box_id, path=path, working_dir=working_dir)

    def rename(self, *, old_path: str, new_path: str, working_dir: Union[str, NotGiven] = NOT_GIVEN) -> FRenameResponse:
        """
        Rename a file or directory.

        Args:
          new_path: New path in the box. If the path does not start with '/', the file/directory
              will be renamed relative to the working directory. If the newPath already
              exists, the rename will fail.

          old_path: Old path in the box. If the path does not start with '/', the file/directory
              will be renamed relative to the working directory. If the oldPath does not
              exist, the rename will fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

        Returns:
            FRenameResponse: The response after renaming.

        Example:
            >>> box.file_system.rename(
            ...     old_path="/path/to/old/file", new_path="/path/to/new/file", working_dir="/path/to/working_dir"
            ... )
        """
        return self.client.v1.boxes.fs.rename(
            box_id=self.box_id, old_path=old_path, new_path=new_path, working_dir=working_dir
        )

    def get(
        self, path: str, *, working_dir: Union[str, NotGiven] = NOT_GIVEN
    ) -> Union["FileOperator", "DirectoryOperator"]:
        """
        Get an operator for a file or directory by its information.

        Args:
            path: Target path in the box. If the path does not start with '/', the file/directory
                will be checked relative to the working directory

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            Union[FileOperator, DirectoryOperator]: The corresponding operator object.

        Example:
            >>> box.file_system.get(path="/path/to/file", working_dir="/path/to/working_dir")
            >>> box.file_system.get("/path/to/file")
        """
        res = self.client.v1.boxes.fs.info(box_id=self.box_id, path=path, working_dir=working_dir)
        if res.type == "file":
            data_file = File(
                path=res.path, type="file", mode=res.mode, name=res.name, size=res.size, lastModified=res.last_modified
            )
            return FileOperator(self.client, self.box_id, data_file)
        else:
            data_dir = Dir(path=res.path, type="dir", mode=res.mode, name=res.name, lastModified=res.last_modified)
            return DirectoryOperator(self.client, self.box_id, data_dir)

    def _data_to_operator(self, data: Optional[Data]) -> Union["FileOperator", "DirectoryOperator"]:
        """
        Convert a Data object to the corresponding operator.

        Args:
            data (Optional[Data]): The data object to convert.
        Returns:
            Union[FileOperator, DirectoryOperator]: The corresponding operator object.
        Raises:
            ValueError: If data is None.
        """
        if data is None:
            raise ValueError("data is None")
        if data.type == "file":
            return FileOperator(self.client, self.box_id, data)
        else:
            return DirectoryOperator(self.client, self.box_id, data)


class FileOperator:
    """
    Operator for file operations within a box.

    Provides methods to read, write, and rename a file.

    Args:
        client (GboxClient): The Gbox client instance.
        box_id (str): The ID of the box to operate on.
        data (DataFile): The file data.
    """

    def __init__(self, client: GboxClient, box_id: str, data: File):
        self.client = client
        self.box_id = box_id
        self.data = data

    def write(self, content: Union[str, FileTypes], *, working_dir: Union[str, NotGiven] = NOT_GIVEN) -> File:
        """
        Write content to this file (text or binary).

        Args:
            content: The content to write to the file.
            working_dir: The working directory to write the file to.

        Returns:
            FWriteResponse: The response after writing.

        Example:
            >>> box.file_system.write(content="Hello, World!", path="/path/to/file", working_dir="/path/to/working_dir")
            >>> box.file_system.write("Hello, World!")
        """
        return self.client.v1.boxes.fs.write(
            box_id=self.box_id,
            content=content,
            path=self.data.path,
            working_dir=working_dir if working_dir else NOT_GIVEN,
        )

    def read(self, *, working_dir: Union[str, NotGiven] = NOT_GIVEN) -> FReadResponse:
        """
        Read the content of this file.

        Args:
            working_dir: The working directory to read the file from.

        Returns:
            FReadResponse: The response containing file content.

        Example:
            >>> box.file_system.read(working_dir="/path/to/working_dir")
            >>> box.file_system.read()
        """
        return self.client.v1.boxes.fs.read(box_id=self.box_id, path=self.data.path, working_dir=working_dir)

    def rename(self, new_path: str, *, working_dir: Union[str, NotGiven] = NOT_GIVEN) -> FRenameResponse:
        """
        Rename this file.

        Args:
            new_path: The new path of the file.
            working_dir: The working directory to rename the file in.

        Returns:
            FRenameResponse: The response after renaming.

        Example:
            >>> box.file_system.rename(new_path="/path/to/new/file", working_dir="/path/to/working_dir")
        """
        return self.client.v1.boxes.fs.rename(
            box_id=self.box_id, old_path=self.data.path, new_path=new_path, working_dir=working_dir
        )


class DirectoryOperator:
    """
    Operator for directory operations within a box.

    Provides methods to list and rename a directory.

    Args:
        client (GboxClient): The Gbox client instance.
        box_id (str): The ID of the box to operate on.
        data (DataDir): The directory data.
    """

    def __init__(self, client: GboxClient, box_id: str, data: Dir):
        self.client = client
        self.box_id = box_id
        self.data = data

    def list_info(
        self, *, depth: Union[float, NotGiven] = NOT_GIVEN, working_dir: Union[str, NotGiven] = NOT_GIVEN
    ) -> FListResponse:
        """
        Get detailed information about files and directories in this directory.

        Args:
            path: Target directory path in the box

            depth: Depth of the directory

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            FListResponse: The response containing file/directory information.

        Example:
            >>> box.file_system.list_info(path="/path/to/directory", depth=1, working_dir="/path/to/working_dir")
            >>> box.file_system.list_info("/path/to/directory")
        """
        return self.client.v1.boxes.fs.list(
            box_id=self.box_id, path=self.data.path, depth=depth, working_dir=working_dir
        )

    def list(
        self, *, depth: Union[float, NotGiven] = NOT_GIVEN, working_dir: Union[str, NotGiven] = NOT_GIVEN
    ) -> List[Union["FileOperator", "DirectoryOperator"]]:
        """
        List files and directories in this directory, returning operator objects.

        Args:
            path: Target directory path in the box

            depth: Depth of the directory

            working_dir: Working directory. If not provided, the file will be read from the
                `box.config.workingDir` directory.

        Returns:
            List[Union[FileOperator, DirectoryOperator]]: List of file or directory operator objects.

        Example:
            >>> box.file_system.list(path="/path/to/directory", depth=1, working_dir="/path/to/working_dir")
            >>> box.file_system.list("/path/to/directory")
        """
        res = self.list_info(depth=depth, working_dir=working_dir)
        result: List[Union["FileOperator", "DirectoryOperator"]] = []
        for r in res.data:
            if r.type == "file":
                file = File(
                    path=r.path, type=r.type, mode=r.mode, name=r.name, size=r.size, lastModified=r.last_modified
                )
                result.append(FileOperator(self.client, self.box_id, file))
            else:
                dir = Dir(path=r.path, type=r.type, mode=r.mode, name=r.name, lastModified=r.last_modified)
                result.append(DirectoryOperator(self.client, self.box_id, dir))
        return result

    def rename(self, *, new_path: str, working_dir: Union[str, NotGiven] = NOT_GIVEN) -> FRenameResponse:
        """
        Rename this directory.

        Args:
          new_path: New path in the box. If the path does not start with '/', the file/directory
              will be renamed relative to the working directory. If the newPath already
              exists, the rename will fail.

          working_dir: Working directory. If not provided, the file will be read from the
              `box.config.workingDir` directory.

        Returns:
            FRenameResponse: The response after renaming.

        Example:
            >>> box.file_system.rename(
            ...     old_path="/path/to/old/file", new_path="/path/to/new/file", working_dir="/path/to/working_dir"
            ... )
            >>> box.file_system.rename("/path/to/old/file", "/path/to/new/file")
        """
        return self.client.v1.boxes.fs.rename(
            box_id=self.box_id, old_path=self.data.path, new_path=new_path, working_dir=working_dir
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["ActionScreenLayoutResponse"]


class ActionScreenLayoutResponse(BaseModel):
    content: str
    """Screen layout content.

    Android boxes (XML):

    ```xml
    <?xml version='1.0' encoding='UTF-8' standalone='yes'?>
    <hierarchy rotation="0">
      <node ... />
    </hierarchy>
    ```

    Browser (Linux) boxes (HTML):

    ```html
    <html>
      <head>
        <title>Example</title>
      </head>
      <body>
        <h1>Hello World</h1>
      </body>
    </html>
    ```
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["MediaVideo"]


class MediaVideo(BaseModel):
    last_modified: datetime = FieldInfo(alias="lastModified")
    """Last modified time of the video"""

    mime_type: str = FieldInfo(alias="mimeType")
    """MIME type of the video"""

    name: str
    """Name of the video"""

    path: str
    """Full path to the video in the box"""

    size: str
    """Size of the video"""

    type: Literal["video"]
    """Video type indicator"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionSettingsUpdateParams"]


class ActionSettingsUpdateParams(TypedDict, total=False):
    scale: Required[float]
    """The scale of the action to be performed.

    Must be greater than 0.1 and less than or equal to 1.

    Notes:

    - Scale does not change the box's actual screen resolution.
    - It affects the size of the output screenshot and the coordinates/distances of
      actions. Coordinates and distances are scaled by this factor. Example: when
      scale = 1, Click({x:100, y:100}); when scale = 0.5, the equivalent position is
      Click({x:50, y:50}).
    """
